<?php
/**
 * This file houses the functions that don't fit in any of the
 * other Extender Pro function files.
 *
 * @package Extender Pro
 */

add_filter( 'body_class', 'extender_pro_body_classes', 15 );
/**
 * Create an array of body classes that reflect various Extender Pro settings.
 *
 * @since 1.0.0
 * @return an array of Extender Pro body classes.
 */
function extender_pro_body_classes( $classes ) {

	if ( is_singular() && get_post_meta( get_the_ID(), '_extender_pro_' . extender_pro_sanitize_string( extender_pro_active_theme_name(), true ) . '_labels', true ) != '' ) {
		
		foreach ( get_post_meta( get_the_ID(), '_extender_pro_' . extender_pro_sanitize_string( extender_pro_active_theme_name(), true ) . '_labels', true ) as $key => $value ) {
			
			$classes[] = 'label-' . $value;
			
		}
		
	}

	$classes[] = 'override';

	return $classes;
	
}

/**
 * Build the Extender Pro select menu options.
 *
 * @since 1.0.0
 */
function extender_pro_build_select_menu_options( $options_array = array(), $selected = '' ) {
	
	foreach( $options_array as $key => $value ) {
		
		$option = '<option value="' . $value . '"';
			
		if ( $value == $selected )
			$option .= ' selected="selected"';

		$option .= '>' . $key . '</option>';
		
		echo $option;
		
	}
	
}

/**
 * Return proper size/unit info (used for image size info).
 *
 * @since 1.0.0
 * @return proper size/unit info.
 */
function extender_pro_format_size_units( $bytes ) {
	
	if ( $bytes >= 1073741824 )
		$bytes = number_format( $bytes / 1073741824, 2 ) . ' GB';
	elseif ( $bytes >= 1048576 )
		$bytes = number_format( $bytes / 1048576, 2 ) . ' MB';
	elseif ( $bytes >= 1024 )
		$bytes = number_format( $bytes / 1024, 2 ) . ' KB';
	elseif ( $bytes > 1 )
		$bytes = $bytes . ' bytes';
	elseif ( $bytes == 1 )
		$bytes = $bytes . ' byte';
	else
		$bytes = '0 bytes';
	
	return $bytes;
	
}

/**
 * Rebuild the multi-image file upload array to be
 * better suited for feeding into the image upload script.
 *
 * @since 1.0.0
 * @return a more usable image upload file array.
 */
function extender_pro_rearray_multi_image_upload( $file_post ) {

	$file_array = array();
	$file_count = count( $file_post['name'] );
	$file_keys = array_keys( $file_post );
	
	for ( $i=0; $i<$file_count; $i++ ) {
		
		foreach ( $file_keys as $key ) {
			
			$file_array[$i][$key] = $file_post[$key][$i];
			
		}
		
	}
	
	return $file_array;
	
}

/**
 * Return either the Parent Theme or Active/Child Theme directory.
 *
 * @since 1.0.0
 * @return specified theme directory.
 */
function extender_pro_get_theme_directory( $parent = true ) {
	
	if ( false == $parent )
		$theme_dir = get_stylesheet_directory();
	else
		$theme_dir = get_template_directory();
	
	return $theme_dir;
	
}

/**
 * Return either the Parent Theme or Active/Child Theme folder name.
 *
 * @since 1.0.0
 * @return specified theme folder name.
 */
function extender_pro_get_theme_folder_name( $parent = true ) {
	
	if ( false == $parent ) {
		
		$theme_folder_name = get_stylesheet();
		
	} else {
		
		$template_dir_explode = explode( '/', get_template_directory() );
		$theme_folder_name = array_pop( $template_dir_explode );
		
	}
	
	return $theme_folder_name;
	
}

/**
 * Check if directory exists and try and create it if it does not.
 *
 * @since 1.0.0
 * @return true or false based on the findings of the function.
 */
function extender_pro_dir_check( $dir ) {
	
	if ( ! is_dir( $dir ) ) {
		
		mkdir( $dir );
		@chmod( $dir, 0755 );
		
	}
	
	if ( is_dir( $dir ) )
		return true;
	else
		return false;
		
}

/**
 * Scan a specified directory and return the names of the directories inside it.
 *
 * @since 1.0.0
 * @return the names of directories inside a specified directory.
 */
function extender_pro_get_dir_names( $dir ) {
	
	if ( ! is_dir( $dir ) )
		return;
	
	$directories = scandir( $dir );
	$directory_array = array();
	
	foreach( $directories as $directory ) {
		
	    if ( $directory === '.' or $directory === '..' )
	    	continue;
	
	    if ( is_dir( $dir . '/' . $directory ) )
			$directory_array[] = $directory;
			
	}
	
	return $directory_array;
	
}

/**
 * This function is not currently in use, but we'll keep it around
 * in case we need it in the future.
 *
 * @since 1.0.0
 */
function extender_pro_copy_dir( $source, $destination ) {
	
	if ( is_dir( $source ) ) {
		
		if ( ! is_dir( $destination ) )
			@mkdir( $destination, 0755, true );

		$handle = opendir( $source );
		while( false !== ( $readdirectory = readdir( $handle ) ) ) {
			
			if ( $readdirectory == '.' || $readdirectory == '..' )
				continue;

			$pathdir = $source . '/' . $readdirectory; 
			if ( is_dir( $pathdir ) ) {
				
				extender_pro_copy_dir( $pathdir, $destination . '/' . $readdirectory );
				continue;
				
			}
			copy( $pathdir, $destination . '/' . $readdirectory );
			
		}
		closedir( $handle );
		
	} else {
		
		copy( $source, $destination );
		
	}
	
}

/**
 * Recursively delete specific folders.
 *
 * @since 1.0.0
 */
function extender_pro_delete_dir( $dir ) {
	
	if ( ! is_dir( $dir ) )
		return;
	
	$handle = opendir( $dir );
	while( false !== ( $file = readdir( $handle ) ) ) {
		
		if ( is_dir( $dir . '/' . $file ) ) {
			
			if ( ( $file != '.' ) && ( $file != '..' ) )
				extender_pro_delete_dir( $dir . '/' . $file );

		} else {
			
			unlink( $dir . '/' . $file );
			
		}
	}
	closedir( $handle );
	rmdir( $dir );
	
}

/**
 * Delete a specified directory and all contents within it
 * and then add the root folder back in.
 *
 * @since 1.0.0
 */
function extender_pro_cleanup_dir( $dir ) {
	
	extender_pro_delete_dir( $dir );
	extender_pro_dir_check( $dir );
	
}

/**
 * Return the entire line of text where a specified string exists.
 *
 * @since 1.0.0
 * @return entire line of text where specified string exists.
 */
function extender_pro_get_line_of_text( $file_contents, $string ) {
	
	$pattern = '/^.*\b' . $string . '\b.*$/m';
	$matches = array();
	preg_match( $pattern, $file_contents, $matches );
	
	return $matches;
	
}

/**
 * Return the contents of a string with specified line replaced.
 *
 * @since 1.0.0
 * @return the contents of a string with specified line replaced.
 */
function extender_pro_replace_line_of_text( $content, $old_line, $new_line ) {

	$new_content = str_replace( $old_line, $new_line, $content );
	
	return $new_content;
	
}

function extender_pro_get_string_between( $string, $start, $end ) {
	
	$string = ' ' . $string;
	$ini = strrpos( $string, $start );
	if ( $ini == 0 )
		return '';
		
	$ini += strlen( $start );
	$len = strrpos( $string, $end, $ini ) - $ini;
	
	return substr( $string, $ini, $len );
	
}

/**
 * Write to a file or create it if it does not exist.
 *
 * @since 1.0.0
 *
 */
function extender_pro_write_file( $path, $code, $stripslashes = true ) {
	
	$handle = @fopen( $path, 'w' );
	
	if ( false == $stripslashes )
		@fwrite( $handle, $code );
	else
		@fwrite( $handle, stripslashes( $code ) );
		
	@fclose( $handle );
	
}

/**
 * Test to see if a directory is writable and/or able to
 * be made writable by Extender Pro.
 *
 * @since 1.0.0
 * @return true or false based on the findings of the function.
 */
function extender_pro_writable( $dir, $level_open = 0777, $level_close = 0755 ) {
	
	if ( ! is_writable( $dir ) )
		@chmod( $dir, $level_open );
	else
		return true;
	
	if ( ! is_writable( $dir ) )
		return false;
	else
		@chmod( $dir, $level_close );
	
	return true;
	
}

/**
 * Minify CSS code.
 *
 * @since 1.0.0
 * @return a minified version of the input CSS.
 */
function extender_pro_minify_css( $css ) {
	
	$css = preg_replace( '/\/\*((?!\*\/).)*\*\//', '', $css );
	$css = preg_replace( '/\s{2,}/', ' ', $css );
	$css = preg_replace( '/\s*([:;{}])\s*/', '$1', $css );
	$css = preg_replace( '/;}/', '}', $css );
	
	return $css;
    
}

/**
 * Minify JS code.
 *
 * @since 1.0.0
 * @return a minified version of the input JS.
 */
function extender_pro_minify_js( $js ) {
	
	$buffer = '';
	while ( list( $idx_start, $keyElement ) = extender_pro_get_next_key_element( $js ) ) {
		
		switch ( $keyElement ) {
			
			case '//':
				
				$idx_start = strpos( $js, '//' );
				$idx_end = strpos( $js, "\n", $idx_start );
				if ( $idx_end !== false )
					$js = substr( $js, 0, $idx_start ) . substr( $js, $idx_end );
				else
					$js = substr( $js, 0, $idx_start );
			
			break;
			case '/*':
				
				$idx_start = strpos( $js, '/*' );
				$idx_end = strpos( $js, '*/', $idx_start )+2;
				$js = substr( $js, 0, $idx_start ) . substr( $js, $idx_end );
				
			break;
			default: // must be handle like string case
			
			$idx_start = extender_pro_get_non_escaped_quote_index( $js, $keyElement );
			if ( strlen( $keyElement ) == 1 ) {
				
				// quote!  Either ' or "
				if ( substr( $js, $idx_start, 1 ) == '\'' )
					$idx_end = extender_pro_get_non_escaped_quote_index( $js, '\'', $idx_start+1 )+1;
				else
					$idx_end = extender_pro_get_non_escaped_quote_index( $js, '"', $idx_start+1 ) +1;
				
				
			} else {
				
				// regex!
				$idx_end = $idx_start + strlen( $keyElement );
				
			}
			$buffer .= extender_pro_minify_javascript_code( substr( $js, 0, $idx_start ) );
			$quote = substr( $js, $idx_start, ( $idx_end-$idx_start ) );
			$quote = str_replace( "\\\n", ' ', $quote );
			$buffer .= $quote;
			$js = substr( $js, $idx_end );
			
		}
		
	}
	
	$buffer .= extender_pro_minify_javascript_code( $js );

	return $buffer;

}

function extender_pro_minify_javascript_code( $js ) {
	
	$blocks = array( 'for', 'while', 'if', 'else' );
	$js = preg_replace( '/([-\+])\s+\+([^\s;]*)/', '$1 (+$2)', $js );
	
	// remove new line in statements
	$js = preg_replace( '/\s+\|\|\s+/', ' || ', $js );
	$js = preg_replace( '/\s+\&\&\s+/', ' && ', $js );
	$js = preg_replace( '/\s*([=+-\/\*:?])\s*/', '$1 ', $js );
	
	// handle missing brackets {}
	foreach( $blocks as $block )
		$js = preg_replace( '/(\s*\b' . $block . '\b[^{\n]*)\n([^{\n]+)\n/i', '$1{$2}', $js );
	
	// handle spaces
	$js = preg_replace( array( "/\s*\n\s*/", "/\h+/" ), array( "\n", " " ), $js ); // \h+ horizontal white space
	$js = preg_replace( array( '/([^a-z0-9\_])\h+/i', '/\h+([^a-z0-9\$\_])/i' ), '$1', $js );
	$js = preg_replace( '/\n?([[;{(\.+-\/\*:?&|])\n?/', '$1', $js );
	$js = preg_replace( '/\n?([})\]])/', '$1', $js );
	$js = str_replace( "\nelse", "else", $js );
	$js = preg_replace( "/([^}])\n/", "$1;", $js );
	$js = preg_replace( "/;?\n/", ";", $js );
	
	return $js;
	
}

function extender_pro_get_next_key_element( $js ) {
	
	$elements = array();
	$keyMarkers = array( '\'', '"', '//', '/*' );
	foreach( $keyMarkers as $marker )
		$elements[$marker] = strpos( $js, $marker );
	
	//regex to detect all regex
	$regex = "/[\k(](\/[\k\S]+\/)/";
	preg_match( $regex, $js, $matches, PREG_OFFSET_CAPTURE, 1 );
	if ( count( $matches ) > 0 )
		$elements[$matches[1][0]] = $matches[1][1];
	
	$elements = array_filter( $elements, function( $k ) { return $k !== false; } );
	if ( count( $elements ) == 0 )
		return false;
		
	$min = min( $elements );
	
	return array( $min, array_keys( $elements, $min )[0] );

}

function extender_pro_get_non_escaped_quote_index( $string, $char, $start = 0 ) {
	
	if ( preg_match( '/(\\\\*)(' . preg_quote( $char ) . ')/', $string, $match, PREG_OFFSET_CAPTURE, $start ) ) {
		
		if ( $match[2][1] == 54 ) { }
		if ( ! isset( $match[1][0]) || strlen( $match[1][0] ) % 2 == 0 )
			return $match[2][1];
		else
			return extender_pro_get_non_escaped_quote_index( $string, $char, $match[2][1] + 1 );
		
	}
	
	return -1;

}

/**
 * Create a Extender Pro Label conditional tag which
 * allows content to be conditionally placed on pages and posts
 * that have specific Extender Pro Labels assigned to them.
 *
 * @since 1.0.0
 */
function extender_pro_has_label( $label = 'label' ) {
	
	$labels_meta_array = get_post_meta( get_the_ID(), '_extender_pro_' . extender_pro_sanitize_string( extender_pro_active_theme_name(), true ) . '_labels', true ) != '' ? get_post_meta( get_the_ID(), '_extender_pro_' . extender_pro_sanitize_string( extender_pro_active_theme_name(), true ) . '_labels', true ) : array();

	if ( is_singular() ) {
		
		if ( in_array( $label, $labels_meta_array ) ) return true;
		
	}

	return false;
	
}

/**
 * Enable Shortcodes in Text Widgets.
 */
add_filter( 'widget_text', 'do_shortcode' );

add_action( 'wp_enqueue_scripts', 'extender_pro_register_scripts' );
/**
 * Register custom scripts.
 *
 * @since 1.0.0
 */
function extender_pro_register_scripts() {
	
	if ( extender_pro_get_settings( 'enable_minified_scripts' ) ) {
		
		wp_register_script( 'custom-minified-header-scripts', extender_pro_get_stylesheet_location( 'url' ) . 'custom-header-scripts.min.js', array( 'jquery' ), filemtime( extender_pro_get_custom_minified_header_scripts_path() ), false );
		wp_register_script( 'custom-minified-footer-scripts', extender_pro_get_stylesheet_location( 'url' ) . 'custom-footer-scripts.min.js', array( 'jquery' ), filemtime( extender_pro_get_custom_minified_footer_scripts_path() ), true );
		
	} else {
		
		wp_register_script( 'custom-header-scripts', extender_pro_get_stylesheet_location( 'url' ) . 'custom-header-scripts.js', array( 'jquery' ), filemtime( extender_pro_get_custom_header_scripts_path() ), false );
		wp_register_script( 'custom-footer-scripts', extender_pro_get_stylesheet_location( 'url' ) . 'custom-footer-scripts.js', array( 'jquery' ), filemtime( extender_pro_get_custom_footer_scripts_path() ), true );
		
	}
	
}

add_action( 'wp_enqueue_scripts', 'extender_pro_load_scripts' );
/**
 * Conditionally load custom scripts.
 *
 * @since 1.0.0
 */
function extender_pro_load_scripts() {
	
	if ( extender_pro_get_settings( 'enable_minified_scripts' ) ) {
		
		if ( file_exists( extender_pro_get_custom_minified_header_scripts_path() ) && 0 != filesize( extender_pro_get_custom_minified_header_scripts_path() ) )
			wp_enqueue_script( 'custom-minified-header-scripts' );
			
		if ( file_exists( extender_pro_get_custom_minified_footer_scripts_path() ) && 0 != filesize( extender_pro_get_custom_minified_footer_scripts_path() ) )
			wp_enqueue_script( 'custom-minified-footer-scripts' );
		
	} else {
		
		if ( file_exists( extender_pro_get_custom_header_scripts_path() ) && 0 != filesize( extender_pro_get_custom_header_scripts_path() ) )
			wp_enqueue_script( 'custom-header-scripts' );
			
		if ( file_exists( extender_pro_get_custom_footer_scripts_path() ) && 0 != filesize( extender_pro_get_custom_footer_scripts_path() ) )
			wp_enqueue_script( 'custom-footer-scripts' );
		
	}
		
}

add_action( 'after_setup_theme', 'extender_pro_require_custom_widget_areas_register_file', 20 );
/**
 * Require the Extender Pro Custom Widget Areas Register file only if it exists.
 *
 * @since 1.0.0
 *
 */
function extender_pro_require_custom_widget_areas_register_file() {
	
	if ( file_exists( extender_pro_get_custom_widget_areas_register_path() ) )
		require_once( extender_pro_get_custom_widget_areas_register_path() );
		
}

add_action( 'after_setup_theme', 'extender_pro_require_custom_widget_areas_file', 20 );
/**
 * Require the Extender Pro Custom Widget Areas file only on the
 * site's front-end and only if it exists.
 *
 * @since 1.0.0
 *
 */
function extender_pro_require_custom_widget_areas_file() {
	
	if ( ! is_admin() && file_exists( extender_pro_get_custom_widget_areas_path() ) )
		require_once( extender_pro_get_custom_widget_areas_path() );
		
}

add_action( 'after_setup_theme', 'extender_pro_require_custom_hook_boxes_file', 20 );
/**
 * Require the Extender Pro Custom Hook Boxes file only on the
 * site's front-end and only if it exists.
 *
 * @since 1.0.0
 *
 */
function extender_pro_require_custom_hook_boxes_file() {
	
	if ( ! is_admin() && file_exists( extender_pro_get_custom_hook_boxes_path() ) )
		require_once( extender_pro_get_custom_hook_boxes_path() );
		
}

add_action( 'after_setup_theme', 'extender_pro_require_custom_functions_files', 20 );
/**
 * Require the Extender Pro Custom Function's files.
 *
 * @since 1.0.0
 *
 */
function extender_pro_require_custom_functions_files() {
	
	if ( file_exists( extender_pro_get_custom_functions_path() ) )
		include_once( extender_pro_get_custom_functions_path() );

	// Include on front-end only.
	if ( file_exists( extender_pro_get_custom_frontend_functions_path() ) && ! is_admin() )
		include_once( extender_pro_get_custom_frontend_functions_path() );
		
}
